package util;


import org.apache.commons.logging.Log;

import org.apache.commons.codec.binary.Base64;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.LogFactory;

import java.io.*;
import java.security.*;
import java.security.cert.Certificate;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.util.Enumeration;

/**
 * <p>
 *
 * </p>
 *
 * @author huaheshang
 * @version 1.0
 * @class_name RSA2Encript
 * @date 2/28/22
 * @time 3:58 PM
 */

public class RSA2Encript {

    final static Log log = LogFactory.getLog(RsaReadUtil.class);

    /**
     * 证书模式+私钥签名
     *
     * @param content    原文
     * @param priKeyPath     私钥路径
     * @param priKeyPass 私钥密码
     * @return 返回签名
     */
    public static String sign(String content, String priKeyPath, String priKeyPass) throws Exception {
        PrivateKey privateKey = getPrivateKeyFromFile(priKeyPath, priKeyPass);
        return doSign(content, privateKey,"UTF-8");
    }

    /**
     * 签名
     *
     * @param content    原文
     * @param privateKey 私钥对象
     * @param charset    编码
     * @return 签名
     * @throws Exception 抛出异常
     */
    protected static String doSign(String content, PrivateKey privateKey, String charset) throws Exception {
        Signature signature;
        signature = Signature.getInstance("SHA256WithRSA");
        signature.initSign(privateKey);
        if (StringUtils.isEmpty(charset)) {
            signature.update(content.getBytes());
        } else {
            signature.update(content.getBytes(charset));
        }

        return byte2Hex(signature.sign());
    }


    /**
     * 公钥验签
     *
     * @param content 原文
     * @param sign    签名
     * @param pubKeyPath  公钥路径
     * @return 验签结果
     * @throws Exception 抛出异常
     */
    public static boolean verify(String content, String sign, String pubKeyPath) {

        try {
            Signature signature;
            PublicKey publicKey = getPublicKeyFromFile(pubKeyPath);
            signature = Signature.getInstance("SHA256WithRSA");
            signature.initVerify(publicKey);
            signature.update(content.getBytes("UTF-8"));
            return signature.verify(hex2Bytes(sign));
        } catch (Exception var6) {
            var6.printStackTrace();
        }
        return false;
    }

    private static String byte2Hex(byte[] srcBytes) {
        StringBuilder hexRetSb = new StringBuilder();
        byte[] arr = srcBytes;
        int len = srcBytes.length;

        for (int i = 0; i < len; ++i) {
            byte b = arr[i];
            String hexString = Integer.toHexString(255 & b);
            hexRetSb.append(hexString.length() == 1 ? 0 : "").append(hexString);
        }

        return hexRetSb.toString();
    }

    private static byte[] hex2Bytes(String source) {
        byte[] sourceBytes = new byte[source.length() / 2];

        for (int i = 0; i < sourceBytes.length; ++i) {
            sourceBytes[i] = (byte) Integer.parseInt(source.substring(i * 2, i * 2 + 2), 16);
        }

        return sourceBytes;
    }

    protected static PrivateKey getPrivateKeyFromFile(String pfxPath, String priKeyPass) {
        FileInputStream priKeyStream = null;

        try {
            priKeyStream = new FileInputStream(pfxPath);
            byte[] reads = new byte[priKeyStream.available()];
            priKeyStream.read(reads);
            PrivateKey var4 = getPrivateKeyByStream(reads, priKeyPass);
            return var4;
        } catch (Exception var14) {
            log.error("解析文件，读取私钥失败:", var14);
        } finally {
            if (priKeyStream != null) {
                try {
                    priKeyStream.close();
                } catch (Exception var13) {
                }
            }

        }

        return null;
    }

    protected static PrivateKey getPrivateKeyByStream(byte[] pfxBytes, String priKeyPass) {
        try {
            KeyStore ks = KeyStore.getInstance("PKCS12");
            char[] charPriKeyPass = priKeyPass.toCharArray();
            ks.load(new ByteArrayInputStream(pfxBytes), charPriKeyPass);
            PrivateKey prikey = null;
            Enumeration<String> aliasEnum = ks.aliases();
            String keyAlias;

            while (aliasEnum.hasMoreElements()) {
                keyAlias = aliasEnum.nextElement();
                prikey = (PrivateKey) ks.getKey(keyAlias, charPriKeyPass);
                if (prikey != null) {
                    break;
                }
            }

            return prikey;
        } catch (IOException var7) {
            log.error("解析文件，读取私钥失败:", var7);
        } catch (KeyStoreException var8) {
            log.error("私钥存储异常:", var8);
        } catch (NoSuchAlgorithmException var9) {
            log.error("不存在的解密算法:", var9);
        } catch (CertificateException var10) {
            log.error("证书异常:", var10);
        } catch (UnrecoverableKeyException var11) {
            log.error("不可恢复的秘钥异常", var11);
        }

        return null;
    }

    protected static PublicKey getPublicKeyFromFile(String pubCerPath) {
        FileInputStream pubKeyStream = null;
        try {
            pubKeyStream = new FileInputStream(pubCerPath);
            byte[] e = new byte[pubKeyStream.available()];
            pubKeyStream.read(e);
            PublicKey var3 = getPublicKeyByText(new String(e));
            return var3;
        } catch (FileNotFoundException var15) {
            log.error("公钥文件路径不存在:", var15);
        } catch (IOException var16) {
            log.error("公钥读取异常:", var16);
        } finally {
            if (pubKeyStream != null) {
                try {
                    pubKeyStream.close();
                } catch (IOException var14) {
                    log.error("公钥读取异常:", var14);
                }
            }

        }
        return null;
    }

    /**
     * 获取公钥对象
     *
     * @param pubKeyText 公钥（字符串）
     * @return 返回公钥对象
     */
    protected static PublicKey getPublicKeyByText(String pubKeyText) {
        try {
            CertificateFactory certificateFactory = CertificateFactory.getInstance("X509");
            BufferedReader br = new BufferedReader(new StringReader(pubKeyText));
            String line;
            StringBuilder keyBuffer = new StringBuilder();

            while ((line = br.readLine()) != null) {
                if (!line.startsWith("-")) {
                    keyBuffer.append(line);
                }
            }

            Certificate certificate = certificateFactory.generateCertificate(new ByteArrayInputStream(Base64.decodeBase64(keyBuffer.toString())));
            return certificate.getPublicKey();
        } catch (Exception var6) {
            log.error("解析公钥内容失败:", var6);
            return null;
        }
    }
}
